﻿using System;
using System.Collections.Generic;
using RMA.OpenNURBS;

namespace CSGLoader
{
    class CSGHeaderReader
    {
        public On3dmProperties fileProperties = new On3dmProperties();
        public List<Guid> fileGuidList = new List<Guid>();

        public bool Read(string filePath)
        {
            using (OnFileHandle file = OnUtil.OpenFile(filePath, "rb"))
            {
                if (file != null)
                {
                    OnBinaryFile bFile = new OnBinaryFile(IOn.archive_mode.read, file);

                    int version3dm = 0;
                    string appName = string.Empty;



                    if (!bFile.Read3dmStartSection(ref version3dm, ref appName))
                    {
                        OnUtil.CloseFile(file);
                        return false;
                    }
                    ;

                    if (fileProperties.m_PreviewImage != null)
                    {
                        fileProperties.m_PreviewImage.Dispose();
                    }

                    if (!bFile.Read3dmProperties(ref fileProperties))
                    {
                        OnUtil.CloseFile(file);
                        return false;
                    }

                    On3dmSettings settings = new On3dmSettings();
                    settings.Default();

                    if (!bFile.Read3dmSettings(ref settings))
                    {
                        OnUtil.CloseFile(file);
                        return false;
                    }

                    Guid readGuid = Guid.NewGuid();
                    if (!bFile.ReadUuid(ref readGuid))
                    {
                        OnUtil.CloseFile(file);
                        return false;
                    }

                    bool validGuid = false;
                    bool displayGroups = false;

                    if (readGuid.CompareTo(new Guid("{A31784F3-23DF-4409-B07F-0417DA142798}")) == 0)
                    {
                        validGuid = true;
                    }

                    // Files with display groups.
                    //{DD8ECCF8-9380-46a8-A177-4BCDE4A6DFBD}
                    if (readGuid.CompareTo(new Guid("{DD8ECCF8-9380-46a8-A177-4BCDE4A6DFBD}")) == 0)
                    {
                        validGuid = true;
                        displayGroups = true;
                    }

                    // Files with Radius MM instead of ring size
                    if (readGuid.CompareTo(new Guid("{847D0609-66E0-4833-8F20-76E61C6A441C}")) == 0)
                    {
                        validGuid = true;
                        displayGroups = true;
                    }

                    // Removed Display Groups..
                    if (readGuid.CompareTo(new Guid("{B237EE6C-AC7E-4725-86EE-54BCCDA34FEA}")) == 0)
                    {
                        validGuid = true;
                    }

                    if (!validGuid)
                    {
                        OnUtil.CloseFile(file);
                        return false;
                    }

                    int year = 0;
                    int month = 0;
                    int day = 0;
                    int fileRevision = 0;

                    if (ReadCSFileInformation(bFile, ref year, ref month, ref day, ref fileRevision))
                    {
                    }

                    if (file != null)
                    {
                        OnUtil.CloseFile(file);
                    }
                }

            }

            return true;
        }

        private bool ReadCSFileInformation(OnBinaryArchive bfile, ref int year, ref int month, ref int day, ref int fileRevision)
        {
            uint typeCode = TypeCodes.TC_USER | TypeCodes.TC_CRC;
            int minor = 0;
            int major = 0;
            if (bfile.BeginRead3dmChunk(typeCode, ref major, ref minor))
            {
                string name = "";
                string company = "";

                bfile.ReadString(ref name);
                bfile.ReadString(ref company);

                bfile.ReadInt(ref year);
                bfile.ReadInt(ref month);
                bfile.ReadInt(ref day);
                bfile.ReadInt(ref fileRevision);


                int fileGuidListCount = 0;
                bfile.ReadInt(ref fileGuidListCount);

                fileGuidList.Clear();

                for (int i = 0; i < fileGuidListCount; i++)
                {
                    Guid g = new Guid();
                    bfile.ReadUuid(ref g);
                    fileGuidList.Add(g);
                }
                

                bfile.EndRead3dmChunk();
                return true;
            }

            return false;
        }


    }
}
